/* USER CODE BEGIN Header */
/**
  ******************************************************************************
  * @file           : main.c
  * @brief          : Main program body
  ******************************************************************************
  * @attention
  *
  * <h2><center>&copy; Copyright (c) 2022 STMicroelectronics.
  * All rights reserved.</center></h2>
  *
  * This software component is licensed by ST under BSD 3-Clause license,
  * the "License"; You may not use this file except in compliance with the
  * License. You may obtain a copy of the License at:
  *                        opensource.org/licenses/BSD-3-Clause
  *
  ******************************************************************************
  */
/* USER CODE END Header */

/* Includes ------------------------------------------------------------------*/
#include "main.h"
#include "dac.h"
#include "dma.h"
#include "i2s.h"
#include "tim.h"
#include "gpio.h"

/* Private includes ----------------------------------------------------------*/
/* USER CODE BEGIN Includes */
#include "utils.h"
#include "jbuf.h"
#include <stdbool.h>
/* USER CODE END Includes */

/* Private typedef -----------------------------------------------------------*/
/* USER CODE BEGIN PTD */

/* USER CODE END PTD */

/* Private define ------------------------------------------------------------*/
/* USER CODE BEGIN PD */
enum { DAC_OFFSET = 32768 };
/* USER CODE END PD */

/* Private macro -------------------------------------------------------------*/
/* USER CODE BEGIN PM */

/* USER CODE END PM */

/* Private variables ---------------------------------------------------------*/

/* USER CODE BEGIN PV */

/* USER CODE END PV */

/* Private function prototypes -----------------------------------------------*/
void SystemClock_Config(void);
/* USER CODE BEGIN PFP */

/* USER CODE END PFP */

/* Private user code ---------------------------------------------------------*/
/* USER CODE BEGIN 0 */
static uint16_t dacSampleTable[2*JBUF_FRAME_SIZE] = { 0 };
static uint16_t i2sData[2*2*JBUF_FRAME_SIZE] = {0}; // L+R channels
static volatile uint16_t *lastI2sData = NULL;
enum { GAIN_MAX = 11 };
int gain = GAIN_MAX;
/* USER CODE END 0 */

/**
  * @brief  The application entry point.
  * @retval int
  */
int main(void)
{
  /* USER CODE BEGIN 1 */

  /* USER CODE END 1 */

  /* MCU Configuration--------------------------------------------------------*/

  /* Reset of all peripherals, Initializes the Flash interface and the Systick. */
  HAL_Init();

  /* USER CODE BEGIN Init */

  /* USER CODE END Init */

  /* Configure the system clock */
  SystemClock_Config();

  /* USER CODE BEGIN SysInit */

  /* USER CODE END SysInit */

  /* Initialize all configured peripherals */
  MX_GPIO_Init();
  MX_DMA_Init();
  MX_DAC1_Init();
  MX_I2S1_Init();
  MX_TIM6_Init();
  /* USER CODE BEGIN 2 */
    HAL_TIM_Base_Start(&htim6);
	if (HAL_DAC_Start_DMA(&hdac1, 0, (uint32_t*) dacSampleTable,
			ARRAY_SIZE(dacSampleTable), DAC_ALIGN_12B_L) != HAL_OK) {
		/* Start DMA Error */
		//TRACE(("Error starting DMA\n"));
		Error_Handler();
	}
	HAL_I2S_Receive_DMA(&hi2s1, i2sData, ARRAY_SIZE(i2sData));
  /* USER CODE END 2 */

  /* Infinite loop */
  /* USER CODE BEGIN WHILE */
  while (1)
  {
    /* USER CODE END WHILE */
        uint32_t now = HAL_GetTick();
        {
            static uint32_t prev = 0;
            if (now - prev >= 1000) {
                prev += 1000;
                HAL_GPIO_TogglePin(LD3_GPIO_Port, LD3_Pin);
                //HAL_GPIO_TogglePin(LD4_GPIO_Port, LD4_Pin);
                //uptimeIncrement();
            }
        }
        {
            static uint32_t prev = 0;
            if (now - prev >= 100) {
                prev += 100;
                static bool pinSet = false;
                if (HAL_GPIO_ReadPin(B1_GPIO_Port, B1_Pin) == GPIO_PIN_SET)
                {
                    if (pinSet == false)
                    {
                        if (gain >= GAIN_MAX)
                            gain = 0;
                        else
                            gain++;
                        pinSet = true;
                    }
                }
                else
                {
                    pinSet = false;
                }
            }
            {
                static uint32_t prev = 0;
                if (now - prev >= 300) {
                    prev += 300;
                    static int ledCnt = 0;
                    if ((ledCnt < (gain+1) * 2) && ((ledCnt % 2) == 0))
                    {
                        HAL_GPIO_WritePin(LD4_GPIO_Port, LD4_Pin, GPIO_PIN_SET);
                    }
                    else
                    {
                        HAL_GPIO_WritePin(LD4_GPIO_Port, LD4_Pin, GPIO_PIN_RESET);
                    }
                    ledCnt++;
                    if (ledCnt > 30)
                        ledCnt = 0;
                }
            }
        }
    #if 1
        static uint16_t *lastProcessedI2sData = NULL;
        if (lastI2sData != lastProcessedI2sData)
        {
            lastProcessedI2sData = (uint16_t*)lastI2sData;
            _Static_assert(ARRAY_SIZE(i2sData) % 4 == 0, "i2sData array size must be divisable by 4 (stereo + double buffering)");
            // these samples are combined from L+R channels, only one channel is active
            // -> pack valid samples into continuous space
            uint16_t *src = lastProcessedI2sData;
            for (uint32_t i=0; i<ARRAY_SIZE(i2sData)/4; i++)
            {
                jbuf_put(*src);
                src += 2;
            }
        }
    #endif
    /* USER CODE BEGIN 3 */
  }
  /* USER CODE END 3 */
}

/**
  * @brief System Clock Configuration
  * @retval None
  */
void SystemClock_Config(void)
{
  RCC_OscInitTypeDef RCC_OscInitStruct = {0};
  RCC_ClkInitTypeDef RCC_ClkInitStruct = {0};

  /** Initializes the CPU, AHB and APB busses clocks
  */
  RCC_OscInitStruct.OscillatorType = RCC_OSCILLATORTYPE_HSI;
  RCC_OscInitStruct.HSIState = RCC_HSI_ON;
  RCC_OscInitStruct.HSICalibrationValue = RCC_HSICALIBRATION_DEFAULT;
  RCC_OscInitStruct.PLL.PLLState = RCC_PLL_ON;
  RCC_OscInitStruct.PLL.PLLSource = RCC_PLLSOURCE_HSI;
  RCC_OscInitStruct.PLL.PLLMUL = RCC_PLL_MUL12;
  RCC_OscInitStruct.PLL.PREDIV = RCC_PREDIV_DIV1;
  if (HAL_RCC_OscConfig(&RCC_OscInitStruct) != HAL_OK)
  {
    Error_Handler();
  }
  /** Initializes the CPU, AHB and APB busses clocks
  */
  RCC_ClkInitStruct.ClockType = RCC_CLOCKTYPE_HCLK|RCC_CLOCKTYPE_SYSCLK
                              |RCC_CLOCKTYPE_PCLK1;
  RCC_ClkInitStruct.SYSCLKSource = RCC_SYSCLKSOURCE_PLLCLK;
  RCC_ClkInitStruct.AHBCLKDivider = RCC_SYSCLK_DIV1;
  RCC_ClkInitStruct.APB1CLKDivider = RCC_HCLK_DIV1;

  if (HAL_RCC_ClockConfig(&RCC_ClkInitStruct, FLASH_LATENCY_1) != HAL_OK)
  {
    Error_Handler();
  }
}

/* USER CODE BEGIN 4 */
static short getSampleVal(short rawValue)
{
    int32_t val = (((int32_t)rawValue) << gain) + DAC_OFFSET;
    if (val < 0)
        val = 0;
    else if (val > 65535)
        val = 65535;
    return val;
}

/**
  * @brief Tx Transfer completed callbacks.
  */
void HAL_DAC_ConvCpltCallbackCh1(DAC_HandleTypeDef* hdac)
{
	//TRACE(("F\n"));
	uint16_t *ptr = dacSampleTable + ARRAY_SIZE(dacSampleTable)/2;
	short *src = jbuf_get();
	for (int i=0; i<ARRAY_SIZE(dacSampleTable)/2; i++) {
		ptr[i] = getSampleVal(src[i]);
	}
}

/**
  * @brief Tx Transfer Half completed callbacks
  */
void HAL_DAC_ConvHalfCpltCallbackCh1(DAC_HandleTypeDef* hdac)
{
	//TRACE(("H\n"));
	uint16_t *ptr = dacSampleTable;
	short *src = jbuf_get();
	for (int i=0; i<ARRAY_SIZE(dacSampleTable)/2; i++) {
		ptr[i] = getSampleVal(src[i]);
	}
}

void HAL_I2S_RxHalfCpltCallback(I2S_HandleTypeDef *hi2s)
{
    _Static_assert(ARRAY_SIZE(i2sData) % 2 == 0, "data array size must be even!");
    lastI2sData = i2sData;
    //HAL_GPIO_TogglePin(LD5_GPIO_Port, LD5_Pin);
}

void HAL_I2S_RxCpltCallback(I2S_HandleTypeDef *hi2s)
{
    _Static_assert(ARRAY_SIZE(i2sData) % 2 == 0, "data array size must be even!");
    lastI2sData = i2sData + (ARRAY_SIZE(i2sData)/2);
    //HAL_GPIO_TogglePin(LD5_GPIO_Port, LD5_Pin);
}
/* USER CODE END 4 */

/**
  * @brief  This function is executed in case of error occurrence.
  * @retval None
  */
void Error_Handler(void)
{
  /* USER CODE BEGIN Error_Handler_Debug */
  /* User can add his own implementation to report the HAL error return state */

  /* USER CODE END Error_Handler_Debug */
}

#ifdef  USE_FULL_ASSERT
/**
  * @brief  Reports the name of the source file and the source line number
  *         where the assert_param error has occurred.
  * @param  file: pointer to the source file name
  * @param  line: assert_param error line source number
  * @retval None
  */
void assert_failed(uint8_t *file, uint32_t line)
{
  /* USER CODE BEGIN 6 */
  /* User can add his own implementation to report the file name and line number,
     tex: printf("Wrong parameters value: file %s on line %d\r\n", file, line) */
  /* USER CODE END 6 */
}
#endif /* USE_FULL_ASSERT */

/************************ (C) COPYRIGHT STMicroelectronics *****END OF FILE****/
